/** @file     logging.h
 *  @author   Adi Singh
 *  @date     August 2019
 */

#ifndef LOGGING_H
#define LOGGING_H

#define PCL_NO_PRECOMPILE

#include <ros/ros.h>
#include <ros/package.h>

#include <pcl/io/pcd_io.h>
#include <pcl/io/io_exception.h>
#include <pcl_ros/point_cloud.h>

#include "toposens_pointcloud/mapping.h"
#include "toposens_msgs/TsPoint.h"


namespace toposens_pointcloud
{

/** @brief Demonstrates basic TF and PCL integrations for TsScan data.
 *  @details Subscribes to a topic publishing TsPoints and maintains an
 *  cummulative pointcloud structure of all points.
 */
class Logging
{
  public:
    /** Subscribes to a #kCloudTopic and prepares a PointCloud structure
     *  for persistent storage.
     *  @param nh Public nodehandle for pub-sub ops on ROS topics.
     *  @param private_nh Private nodehandle for accessing launch parameters.
     *  @param pcd_save_rate defines save interval of cumulative pointcloud.
     */
    Logging(ros::NodeHandle nh, ros::NodeHandle private_nh);

    /** Destructor frees memory from heap */
    ~Logging();

    /** Saves all pointcloud data contained in #store as a PCD file whose path
     *  is specified by #pcd_path.
     *  @param  event  TimerEvent, currently ignored.
     */
    void save(const ros::TimerEvent& event);

    /** Self-contained Logging class handler that can be threaded.
     *  @param pcd_save_rate defines save interval of cumulative pointcloud.
     */
    static void logPointcloud(){

      ros::NodeHandle nh;
      ros::NodeHandle private_nh("~");

      Logging l(nh, private_nh);

      while(true){
        try
        {
          boost::this_thread::sleep(boost::posix_time::milliseconds(10));
        }
        catch(boost::thread_interrupted&)
        {
          return;
        }
      }
    }


  private:
    /** Waits for pointcloud updates to become available and accumulates them
     *  in a perisistent pointcloud structure.
     *  @param  msg  Pointer to an incoming TsScan message.
     */
    void _accumulate(const XYZINCloud::ConstPtr& msg);

    std::string target_frame;   /**< Target frame for scan transformations.*/

    XYZINCloud::Ptr store;      /**< Collection of all pointclouds from a single run.*/
    std::string pcd_path;       /**< Path of file in which pointcloud is stored by #save().*/

  	ros::Subscriber _cloud_sub; /**< Handler for subscribing to TsPoints.*/
    ros::Timer timer;           /**< Timer for pointcloud saving interval. */

    boost::mutex _store_mutex;  /**< Mutex for locking pointcloud.*/
};

} // namespace toposens_pointcloud

#endif
